#include <app/Application.h>
#include <interface/Alert.h>
#include <InterfaceKit.h>
#include <FindDirectory.h>
#include <stdio.h>

#include "Wind.h"

#define MENU_HEIGHT 15

// ----------------------------------------------------------------------
// OpenGL drawing thread

void ShowMsg(char *msg)
{
	BAlert *alert = new BAlert("Message",msg,"OK");
	alert->Go();
}


int32 Wind::render_thread( void *data )
{
	Wind *wnd = (Wind *)data;

	wnd->MakeCurrent();
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( -20, 20, -20, 20, -30, 30 );
	glMatrixMode( GL_MODELVIEW );

	glClearColor( 0.f, 0.f, 0.f, 1.f );
	
	// Enable Z-BUFFER
	glEnable( GL_DEPTH_TEST );
	
	while( !wnd->window_teardown )
	{

		glRotatef( 0.5, 1.0, 1.0, 0.0 );
		glBegin(GL_TRIANGLES);
						glColor3f(0,0,1);
			glVertex3f(-10,20,0);
						glColor3f(0,1,0);
			glVertex3f(-20,-20,0);
						glColor3f(1,0,0);
			glVertex3f(20,-10,0);
		glEnd();

		// use DOUBLE BUFFER. SYNC = TRUE (enable)
		wnd->SwapBuffers( wnd->vsync );
		glDrawBuffer( GL_BACK );

		wnd->YieldCurrent();

		// clear COLOR and Z-BUFFER
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	}
	
	wnd->ReleaseCurrent();
}

void Wind::DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum )
{
	char buf[256];
	BMessage msg ('addm');
	msg.AddInt32("num",device_id);
	msg.what = 'dev';
	sprintf( buf, "%s  mon=%x", name, monitor );
	BMenuItem *item = new BMenuItem( buf, new BMessage(msg));
	DeviceMenu->AddItem( item );
	if( device_id == BGL_DEVICE_SOFTWARE )
	{
		item->SetMarked( true );
		checkedDevice = item;
	}
	
	currentDevice++;
	devices[currentDevice].deviceID = device_id;
	devices[currentDevice].first = 0;
}


// ----------------------------------------------------------------------
// Constructor and destructor

Wind::Wind( const char *title )
	: BDirectGLWindow( BRect( 100.0, 100.0, 499.0, 499.0 ), title,
			   		 B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
			   		 B_ASYNCHRONOUS_CONTROLS | B_NOT_ZOOMABLE )
{
	window_teardown = false;	// drawing thread control variable
	currentDevice = -1;			// current DEVICE number
	draw_thread = -1;			// drawing thread ID
	initialized = false;
	vsync = false;

	// Create a menu
	BRect r(0,0,0,MENU_HEIGHT);
	BMenuBar *mb;
	mb = new BMenuBar(r,"main menu");
	AddChild( mb );
	BMessage msg ('addm');

	msg.ReplaceInt32("num",256);
	msg.what = 'dev';
	mb->AddItem( DeviceMenu = new BMenu("Device") );
	mb->ResizeToPreferred();

	// Gets Available Devices
	EnumerateDevices( BGL_MONITOR_ANY, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE );
	
	// Restarts OPENGL
	if( RestartGL( BGL_DEVICE_SOFTWARE ) < B_OK ) return;
	
	Show();
	
	ShowMsg("Use SPACE to En/Disable VSYNC");
}

Wind::~Wind( void )
{
	status_t ret;

	Hide();
	Sync();
	window_teardown = true;
	wait_for_thread( draw_thread, &ret );
}


status_t Wind::RestartGL( uint32 deviceID )
{
	status_t ret;

	window_teardown = true;
	if( draw_thread >= 0 )
		wait_for_thread( draw_thread, &ret );

	if( initialized )
	{
		ShutdownGL();
		initialized = false;
	}
	initialized = true;

	window_teardown = false;

	// Start the drawing thread.
	draw_thread = spawn_thread( render_thread, "OpenGL drawing thread", B_DISPLAY_PRIORITY, this );
	if( draw_thread < 0 )
	{
		be_app->PostMessage( B_QUIT_REQUESTED );
		return B_ERROR;
	}

	fullscreenEnabled = false;
	if( InitializeGL( deviceID, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE ) < B_OK )
	{
		be_app->PostMessage( B_QUIT_REQUESTED );
		exit(0);  // Fix me
	}

	BMessage msg ('addm');
	msg.AddInt32("XxY",0);
	msg.what = 'res';
	int32 ct;
	for( ct=0; ct<MAX_DEVICES; ct++ )
	{
		if( devices[ct].deviceID == deviceID )
			break;
	}

	resume_thread( draw_thread );
	return B_OK;
}


bool Wind::QuitRequested( void )
{
	be_app->PostMessage( B_QUIT_REQUESTED );
	return false;
}

void Wind::MessageReceived( BMessage *msg )
{
	BMenuItem *i;
	switch( msg->what )
	{
	case 'dev':
		{
			msg->FindPointer("source", (void **)&i);
			checkedDevice->SetMarked( false );
			checkedDevice = i;
			checkedDevice->SetMarked( true );
	
			uint32 new_mode = msg->FindInt32( "num" );
			if( RestartGL( new_mode ) < B_OK )
				exit(0);
		}
		break;

	case B_KEY_DOWN:
		if( RestartGL( 1 ) < B_OK ) exit(0);
		
		vsync = !vsync;
		
		if( fullscreenEnabled )
		{
			fullscreenEnabled = false;
			FullscreenDisable();
			ResizeTo( fullscreenWidth-1, fullscreenHeight-1 );
			break;
		}
		paused = !paused;
	
	default:
		BDirectWindow::MessageReceived( msg );
		break;
	}
}


void Wind::FrameResized( float width, float height )
{
	MakeCurrent();
	if( fullscreenEnabled )
	{
		glViewport( 0, 0, fullscreenWidth, fullscreenHeight );
	}
	else
	{
		if( height > MENU_HEIGHT )
			glViewport( 0, 0, int(width-1), int(height-1) - MENU_HEIGHT );
	}
	ReleaseCurrent();
}

BRect Wind::UpdateGLRect( BRect window_rect )
{
	BRect r;

	if( fullscreenEnabled )
	{
		r = BRect( 0, 0, fullscreenWidth-1, fullscreenHeight-1 );
	}
	else
	{
		r = window_rect;
		if( r.bottom > MENU_HEIGHT )
			r.top += MENU_HEIGHT;
		else
			r.bottom = 0;
	}
	return r;
}



